#include<linux/module.h>
#include<linux/kernel.h>
#include<linux/errno.h>
#include<linux/mm.h>
#include<linux/init.h>
#include<linux/pci.h>

#define MAX_DEV 8
static char * fb[MAX_DEV] = {
	NULL,NULL,NULL,NULL,
	NULL,NULL,NULL,NULL,
};

static struct pci_device_id smi_pci_table[] = {
	{0x126f,0x0501,PCI_ANY_ID,PCI_ANY_ID,0,0,0},
	{0,}
};

static int __devinit wc_probe(struct pci_dev * pdev,const struct pci_device_id * ent)
{
	static int index = 0;
	void * va;
	printk("enter probe\n");
	/* for sm502,8mb is the only option */
	va = ioremap_wc(pci_resource_start(pdev,0),0x800000);
	if(!va){
		printk("ioremap_wc failed\n");
	}else{
		printk("mapped address = %p\n",va);
		fb[index++] = va;
	}
	return 0;
}

static void __devexit wc_remove(struct pci_dev * pdev)
{
	int i;
	for(i=0;i<MAX_DEV;i++){
		if(fb[i] != NULL){
			iounmap(fb[i]);
			fb[i] = NULL;
		}
	}			
}

static struct pci_driver wc_driver = {
	.name = "smi_wcpatch",
	.id_table = smi_pci_table,
	.probe = wc_probe,
	.remove = __devexit_p(wc_remove),
};

static int __init wc_init(void)
{
	printk("enter init\n");
	return pci_register_driver(&wc_driver);
}

#ifdef MODULE
static void __exit wc_exit(void)
{
	pci_unregister_driver(&wc_driver);
}

module_exit(wc_exit)
#endif

module_init(wc_init)
MODULE_AUTHOR("monk liu<monk.liu@siliconmotion.com>");
MODULE_DESCRIPTION("Open sm502 wc map capability");
MODULE_LICENSE("Dual BSD/GPL");
MODULE_DEVICE_TABLE(pci,smi_pci_table);


